from aestate.util import others, sqlOpera
from aestate.util.Log import ALog

alog = ALog
Alog = ALog
timeUtil = others
timeutil = others
cov = others
pack = others

sqlText = sqlOpera.TextUtil
sql_text = sqlOpera.TextUtil
sqltext = sqlOpera.TextUtil
textUtil = sqlOpera.TextUtil
textutil = sqlOpera.TextUtil
text_util = sqlOpera.TextUtil
